searchNodes=[{"ref":"Specify.html","title":"Specify","type":"module","doc":"Specify allows you to make your configuration explicit: Specify exactly what fields are expected. Specify exactly what values these fields might take, by giving them parser-functions. Load their values from a slew of different locations, with &#39;explicitly passed in to the function&#39; as final option."},{"ref":"Specify.html#defconfig/2","title":"Specify.defconfig/2","type":"macro","doc":"Defines a configuration structure in the current module. Fields are added to this configuration structure by calling Specify.Schema.field/3 (which can be called just as field because Specify.Schema is autoamatically imported into the inner context of the call to defconfig.) The options that can be passed to this module are used as defaults for the options passed to a call to Specify.load/2 or YourModule.load/1. See also Specify.Schema.field/3 and Specify.Options Reflection The special function __specify__/1 will be defined on the module as well. It is not intended to be used by people that want to consume your configuration, but it is there to e.g. allow Specify.Provider implementations to be smarter in how they fetch the configuration for the module. For instance, configuration might be lazily fetched, when knowing what field names exist beforehand. YourModule.__specify__/1 supports the following publicly usable parameters: __specify__(:field_names) returns a MapSet of atoms, one per field in the configuration structure. __specify__(:defaults) returns a Map containing only the field_name =&gt; values of field names having default values. __specify__(:requireds) returns a MapSet of atoms, one per required field in the configuration structure. __specify__(:parsers) returns a Map of the format field_name =&gt; parser. __specify__(:field_options) returns a Map of the format field_name =&gt; options, where options is the keyword-list that was passed to the field macro."},{"ref":"Specify.html#load/2","title":"Specify.load/2","type":"function","doc":"Loads, parses, and normalizes the configuration of config_module, based on the current source settings, returning the result as a struct. (This is the more general way of calling config_module.load/1). See Specify.Options for more information of the options that can be supplied to this function, and how it can be configured further."},{"ref":"Specify.html#load_explicit/3","title":"Specify.load_explicit/3","type":"function","doc":"Loads, parses and normalizes the configuration of config_module, using the provided explicit_values (and falling back to values configured elsewhere) This call is conceptually the same as Specify.load(config_module, [explicit_values: [] | options]), but makes it more explicit that values are meant to be passed in as arguments. Prefer this function if you do not intend to use Specify&#39;s &#39;cascading configuration&#39; functionality, such as when e.g. just parsing options passed to a function, use-statement or other macro."},{"ref":"Specify.MissingRequiredFieldsError.html","title":"Specify.MissingRequiredFieldsError","type":"exception","doc":"Default exception to be raised when a required field is not existent in any configuration source."},{"ref":"Specify.Options.html","title":"Specify.Options","type":"module","doc":"This struct represents the options you can pass to a call of Specify.load/2 (or YourModule.load/1). Metaconfiguration Besides making it nice and explicit to have the options listed here, Specify.Options has itself been defined using Specify.defconfig/2, which means that it (and thus what default options are passed on to to other Specify configurations) can be configured in the same way. Configuration structure documentation: This configuration was made using the Specify library. It contains the following fields: sources A list of structures that implement the Specify.Provider protocol, which will be used to fetch configuration from. Later entries in the list take precedence over earlier entries. Defaults always have the lowest precedence, and :explicit_values always have the highest precedence. Validated/parsed by calling &amp;Specify.Options.list_of_sources/1. Defaults to []. explicit_values A list or map (or other enumerable) representing explicit values that are to be used instead of what can be found in the implicit sources stack. Validated/parsed by calling &amp;Specify.Parsers.term/1. (Specified as :term) Defaults to []. missing_fields_error The error to be raised if a missing field which is required has been encountered. Validated/parsed by calling &amp;Specify.Parsers.term/1. (Specified as :term) Defaults to Specify.MissingRequiredFieldsError. parsing_error The error to be raised if a field value could not properly be parsed. Validated/parsed by calling &amp;Specify.Parsers.term/1. (Specified as :term) Defaults to Specify.ParsingError. explain When set to true, rather than returning the config struct, a map is returned with every field-key containing a list of consecutive found values. This is useful for debugging. Validated/parsed by calling &amp;Specify.Parsers.boolean/1. (Specified as :boolean) Defaults to false. Metaconfiguration Gotcha&#39;s Specify will only be able to find a source after it knows it exists. This means that it is impossible to define a different set of sources inside an external source. For this special case, Specify will look at the current process&#39; Process dictionary, falling back to the Application environment (also known as the Mix environment), and finally falling back to an empty list of sources (its default). So, from lowest to highest precedence, option values are loaded in this order: Specify.Options default Application Environment :specify Process Dictionary :specify field Options passed to Comfy.defconfig Options passed to YourModule.load Requiring Specify to be configured in such a general way seems highly unlikely. If the current approach does turn out to not be good enough for your use-case, please open an issue on Specify&#39;s issue tracker."},{"ref":"Specify.Options.html#list_of_sources/1","title":"Specify.Options.list_of_sources/1","type":"function","doc":""},{"ref":"Specify.Options.html#load/1","title":"Specify.Options.load/1","type":"function","doc":"Loads, parses, and normalizes the configuration of Specify.Options, based on the current source settings, returning the result as a struct. For more information about the options this function supports, see Specify.load/2 and Specify.Options"},{"ref":"Specify.Options.html#load_explicit/2","title":"Specify.Options.load_explicit/2","type":"function","doc":"Loads, parses and normalizes the configuration of Specify.Options, using the provided explicit_values (and falling back to values configured elsewhere) For more information about the options this function supports, see Specify.load_explicit/3 and Specify.Options"},{"ref":"Specify.Parsers.html","title":"Specify.Parsers","type":"module","doc":"Simple functions to parse strings to datatypes commonly used during configuration. These functions can be used as parser/validator function in a call to Specify.Schema.field, by using their shorthand name (:integer as shorthand for &amp;Specify.Parsers.integer/1). (Of course, using their longhand name works as well.) Defining your own parser function A parser function receives the to-be-parsed/validated value as input, and should return {:ok, parsed_val} on success, or {:error, reason} on failure. Be aware that depending on where the configuration is loaded from, the to-be-parsed value might be a binary string, or already the Elixir type you want to convert it to."},{"ref":"Specify.Parsers.html#atom/1","title":"Specify.Parsers.atom/1","type":"function","doc":"Parses an atom or a binary string representing an (existing) atom. Will not create new atoms (See String.to_existing_atom/1 for more info)."},{"ref":"Specify.Parsers.html#boolean/1","title":"Specify.Parsers.boolean/1","type":"function","doc":"Parses a boolean or a binary string representing a boolean value, turning it into a boolean."},{"ref":"Specify.Parsers.html#float/1","title":"Specify.Parsers.float/1","type":"function","doc":"Parses a float and turns a binary string representing a float into an float. Will also accept integers, which are turned into their float equivalent."},{"ref":"Specify.Parsers.html#integer/1","title":"Specify.Parsers.integer/1","type":"function","doc":"Parses an integer and turns binary string representing an integer into an integer."},{"ref":"Specify.Parsers.html#list/2","title":"Specify.Parsers.list/2","type":"function","doc":"Parses a list of elements. In the case a binary string was passed, this parser uses Code.string_to_quoted under the hood to check for Elixir syntax, and will only accepts binaries representing lists. If a list was passed in (or after turning a binary into a list), it will try to parse each of the elements in turn."},{"ref":"Specify.Parsers.html#string/1","title":"Specify.Parsers.string/1","type":"function","doc":"Parses a binary string and turns anything that implements String.Chars into its binary string representation by calling to_string/1 on it."},{"ref":"Specify.Parsers.html#term/1","title":"Specify.Parsers.term/1","type":"function","doc":"Accepts any Elixir term as-is. Will not do any parsing. Only use this as a last resort. It is usually better to create your own dedicated parsing function instead."},{"ref":"Specify.Parsers.html#unsafe_atom/1","title":"Specify.Parsers.unsafe_atom/1","type":"function","doc":"Parses an atom or a binary string representing an (potentially not yet existing!) atom. Will create new atoms. Whenever possible, consider using atom/1 instead. (See String.to_atom/1 for more info on why creating new atoms is usually a bad idea)."},{"ref":"Specify.ParsingError.html","title":"Specify.ParsingError","type":"exception","doc":"Default exception to be raised when it is impossible to parse one of the configuration values. (See also Specify.Parsers)"},{"ref":"Specify.Provider.html","title":"Specify.Provider","type":"protocol","doc":"Protocol to load configuration from a signle source. Configuration Providers implement this protocol, which consists of only one function: load/2."},{"ref":"Specify.Provider.html#load/2","title":"Specify.Provider.load/2","type":"function","doc":"Loads the configuration of specification module from the source indicated by struct. Its first argument is the implementation&#39;s own struct, the second argument being the configuration specification&#39;s module name. If extra information is required about the configuration specification to write a good implementation, the Reflection function module_name.__specify__ can be used to look these up. See also Specify.defconfig/2 and Specify.Options."},{"ref":"Specify.Provider.html#t:t/0","title":"Specify.Provider.t/0","type":"type","doc":""},{"ref":"Specify.Provider.MixEnv.html","title":"Specify.Provider.MixEnv","type":"module","doc":"A Configuration Provider source based on Mix.env() / Application.get_env/2. Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom end end"},{"ref":"Specify.Provider.MixEnv.html#new/2","title":"Specify.Provider.MixEnv.new/2","type":"function","doc":"By default, will try to use Application.get_all_env(YourConfigModule) to fetch the source&#39;s configuration. A different application name can be used by supplying a different application argument. If the actual configuration is only inside one of the keys in this application, the second field key can also be provided. iex&gt; Application.put_env(Elixir.Pet, :name, &quot;Timmy&quot;) iex&gt; Application.put_env(Elixir.Pet, :kind, &quot;cat&quot;) iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new()]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new(Elixir.Pet)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Application.put_env(:second_pet, :name, &quot;John&quot;) iex&gt; Application.put_env(:second_pet, :kind, :dog) iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new(:second_pet)]) %Pet{name: &quot;John&quot;, kind: :dog}"},{"ref":"Specify.Provider.Process.html","title":"Specify.Provider.Process","type":"module","doc":"A Configuration Provider source based on the current process&#39; Process Dictionary. Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom end end"},{"ref":"Specify.Provider.Process.html#new/1","title":"Specify.Provider.Process.new/1","type":"function","doc":"By default, will try to use Process.get(YourModule) to fetch the source&#39;s configuration. A different key can be used by supplying a different key argument. iex&gt; Process.put(Pet, %{name: &quot;Timmy&quot;, kind: :cat}) iex&gt; Pet.load(sources: [Specify.Provider.Process.new(Pet)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Process.put(:another_pet, %{name: &quot;John&quot;, kind: :dog}) iex&gt; Pet.load(sources: [Specify.Provider.Process.new(:another_pet)]) %Pet{name: &quot;John&quot;, kind: :dog}"},{"ref":"Specify.Provider.SystemEnv.html","title":"Specify.Provider.SystemEnv","type":"module","doc":"A Configuration Provider source based on System.get_env/2 Values will be loaded based on \#{prefix}_\#{capitalized_field_name}. prefix defaults to the capitalized name of the configuration specification module. capitalized_field_name is in CONSTANT_CASE (all-caps, with underscores as word separators). Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom end end iex&gt; System.put_env(&quot;PET_NAME&quot;, &quot;Timmy&quot;) iex&gt; System.put_env(&quot;PET_KIND&quot;, &quot;cat&quot;) iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new()]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new(&quot;PET&quot;)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; System.put_env(&quot;SECOND_PET_NAME&quot;, &quot;John&quot;) iex&gt; System.put_env(&quot;SECOND_PET_KIND&quot;, &quot;dog&quot;) iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new(&quot;SECOND_PET&quot;)]) %Pet{name: &quot;John&quot;, kind: :dog}"},{"ref":"Specify.Provider.SystemEnv.html#new/1","title":"Specify.Provider.SystemEnv.new/1","type":"function","doc":""},{"ref":"Specify.Schema.html","title":"Specify.Schema","type":"module","doc":"Functions that can be used inside Specify.defconfig/2."},{"ref":"Specify.Schema.html#field/3","title":"Specify.Schema.field/3","type":"macro","doc":"Specifies a field that is part of the configuration struct. Can/should only be called inside a call to Specify.defconfig. name should be an atom representing the field. It will also become the field name for the struct that is created. parser should either be: an arity-one function capture like &amp;YourModule.some_type_parser/1. An atom representing one of the common parser function names in Specify.Parsers like :integer, :string, :boolean or :term. A two-element tuple like {:list, :atom}. The first element represents the &#39;collection parser&#39; which is an arity-2 function that takes the &#39;element parser&#39; as second argument. The second element is the &#39;element parser&#39;. Both of the elements listed in the tuple can also be either an atom, or a function capture with the correct arity. (like {&amp;YourAwesomeModule.fancy_collection/2, :integer}). Supported field options are: default:, supplies a default value to this field. If not set, the configuration field is set to be required. You are highly encouraged to add a @documentation text above each and every field; these will be added to the configuration&#39;s module documentation."}]